/*
 * Copyright (c) 2002 - trem0r
 * This file is part of the PornView - photo collection manager.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_MPLAYER

#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>

#include "gtkmplayer.h"

/* forward declarations */
void    change_size (GtkWidget * widget,
		     GtkAllocation * allocation, gpointer user_data);

void    gtk_mplayer_restart (GtkMPlayer * player);

/* constructor */
GtkMPlayer *
gtk_mplayer_new (gchar * file)
{
    GtkMPlayer *p;
    GtkWidget *w;
    GtkWidget *s;
    gchar  *f;

    p = g_new0 (GtkMPlayer, 1);
    w = gtk_vbox_new (FALSE, 0);
    s = gtk_socket_new ();
    f = g_strdup (file);

    gtk_signal_connect (GTK_OBJECT (w), "size-allocate", change_size, p);
    gtk_container_add (GTK_CONTAINER (w), s);

    p->widget = w;
    p->mySocket = s;
    p->file = f;
    p->ready = FALSE;
    p->childpid = -1;
    p->timer = 0;
    return p;
}

/* waits for size changes and restarts */
void
change_size (GtkWidget * widget,
	     GtkAllocation * allocation, gpointer user_data)
{
    GtkMPlayer *p = (GtkMPlayer *) user_data;

    p->width = allocation->width;
    p->height = allocation->height;
    if (p->ready)
	gtk_mplayer_restart (p);

}

/* return the integrable widget */
GtkWidget *
gtk_mplayer_get_widget (GtkMPlayer * player)
{
    return player->widget;
}

/* init */
void
gtk_mplayer_init (GtkMPlayer * player)
{
    gtk_widget_realize (player->mySocket);
    player->xid = GDK_WINDOW_XWINDOW (player->mySocket->window);
    gtk_widget_show (player->mySocket);

}

/* show */
void
gtk_mplayer_show (GtkMPlayer * player)
{
    gtk_widget_show (player->widget);
}

/* start reading */
void
gtk_mplayer_start (GtkMPlayer * player)
{
    player->ready = TRUE;
}

/* helper to launch mplayer
 * Thanks to the GNU's system manpage ;-)
 */
int
my_system (GtkMPlayer * player, const char *command)
{
    int     pid, status;
    extern char **environ;
    if (command == 0)
	return 1;
    pid = fork ();
    if (pid == -1)
	return -1;
    if (pid == 0)
    {
	char   *argv[4];
	argv[0] = "sh";
	argv[1] = "-c";
	argv[2] = g_strdup (command);
	argv[3] = 0;
	execve ("/bin/sh", argv, environ);
	exit (127);
	g_free (argv[0]);
	g_free (argv[1]);
	g_free (argv[2]);
	g_free (argv[3]);
    }
    do
    {
	if (waitpid (pid, &status, 0) == -1)
	{
	    if (errno != EINTR)
		return -1;
	}
	else
	{
	    player->childpid = pid + 1;	/*FIXME that's a hack, race condition could occur ? */
	    return status;
	}
    }
    while (1);
}

/* stop player */
void
gtk_mplayer_stop (GtkMPlayer * player)
{
    if (player->childpid > 0)
    {
	kill (player->childpid, SIGKILL);
	kill (player->childpid + 1, SIGKILL);
	player->childpid = -1;
    }
}

/* check for end of film (and relaunch mplayer) */
gint
check_pid (gpointer data)
{
    GtkMPlayer *player = (GtkMPlayer *) data;
    gchar  *path = g_strdup_printf ("/proc/%d", player->childpid);
    FILE   *fp = NULL;
    player->timer = gtk_timeout_add (1000, check_pid, player);
    if ((fp = fopen (path, "r")) == NULL)
	gtk_mplayer_restart (player);
    else
	fclose (fp);
    g_free (path);

}

/* check if player runs */

gboolean
gtk_mplayer_is_running (GtkMPlayer * player)
{
    if (!player->ready && player->childpid > 0)
	return TRUE;
    return FALSE;
}

/* restart film */
void
gtk_mplayer_restart (GtkMPlayer * player)
{
    gchar  *cmd;
    if (gtk_mplayer_is_running (player))
    {
	gtk_mplayer_stop (player);
	player->ready = TRUE;
    }
    if (player->ready)
    {
	if (player->timer)
	{
	    gtk_timeout_remove (player->timer);
	    player->timer = 0;
	}

	cmd =
	    g_strdup_printf ("mplayer -vo xv -vop scale=%d:-3 -wid %d %s &",
			     player->width, player->xid, player->file);
	player->ready = FALSE;
	player->timer = gtk_timeout_add (1000, check_pid, player);
	my_system (player, cmd);
	g_free (cmd);
    }
}

void
gtk_mplayer_open_file (GtkMPlayer * player, gchar * file)
{
    player->file = g_strdup (file);
    if (gtk_mplayer_is_running (player))
    {
	gtk_mplayer_restart (player);
    }
}

#endif /* ENABLE_MPLAYER */
